#include <stdio.h>
#include "slog_profile.h"              /*I "slog_profile.h" I*/



/*
    Replace the specified string's all src_char's by the dest_char
*/
char *str_replace( char *str, const char src_char, const char dest_char )
{
    char *cptr;
    for ( cptr = str; *cptr != '\0'; cptr ++ ) {
       if ( *cptr == src_char )
           *cptr = dest_char;
    }

    return str;
}



/*
    trim all the leading and trailing non-alphanumeric and non-underscore
    characters in the given string
*/
char *str_trim( char *str )
{
    static char  *valid_chars = "abcdefghijklmnopqrstvvwxyz"
                                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                "0123456789_";
           char  *str_beg, *str_end;
           int    str_lgth;
           int    ierr;

    /*  Locate the 1st valid char  */
    str_beg = strpbrk( str, valid_chars );
    /*  Locate the last valid char  */
    if ( str_beg != NULL ) {
        /*  str_end = strrpbrk( str, valid_chars );  */
        for (  str_end  = str_beg + strlen( str_beg ) - 1;
              *str_end != '\0'; str_end-- ) {
            if ( isalnum( *str_end ) || *str_end == '_' )
                break;
        }
    }

    if ( str_end != NULL ) {
        str_lgth = str_end - str_beg + 1;
        if ( str_beg[ str_lgth ] != '\0' ) str_beg[ str_lgth ] = '\0';
    }

    return str_beg;
}



void SLOG_PROF_Assign(       SLOG_intvlinfo_t        *intvlinfo,
                       const SLOG_intvltype_t         in_intvltype,
                       const char *                   in_classtype,
                       const char *                   in_label,
                       const char *                   in_color )
{
    const char          delimiter = '\t';
    const char          repl_char = ' ';
    const char          null_char = '\0';
          SLOG_uint32   lgth;

    intvlinfo->intvltype = in_intvltype;

    /*  The last 2 characters of char[SLOG_STRING_LEN] are null & delimiter  */
    lgth = SLOG_STRING_LEN - 2;
    strncpy( intvlinfo->classtype, in_classtype, lgth );
    intvlinfo->classtype[ lgth ] = null_char;
    str_trim( intvlinfo->classtype );
    str_replace( intvlinfo->classtype, delimiter, repl_char ); 

    strncpy( intvlinfo->label,     in_label,     lgth );
    intvlinfo->label[ lgth ] = null_char;
    str_trim( intvlinfo->label );
    str_replace( intvlinfo->label, delimiter, repl_char );

    strncpy( intvlinfo->color,     in_color,     lgth );
    intvlinfo->color[ lgth ] = null_char;
    str_trim( intvlinfo->color );
    str_replace( intvlinfo->color, delimiter, repl_char );
}



int SLOG_PROF_IsDuplicated( const SLOG_prof_t       *profile,
                            const SLOG_intvlinfo_t  *in_info )
{
    SLOG_intvlinfo_t  *cur_info;
    int                ii;
    int                NotMatched = 1;

    for ( ii = 0; NotMatched && (ii < profile->Nentries); ii++ ) {
        cur_info = &( profile->entries[ ii ] );
        NotMatched = ! SLOG_IntvlInfo_IsKeyEqualTo( cur_info, in_info );
    }

    return ! NotMatched;
}


/*@C
    SLOG_PROF_Open - Creates the SLOG Display Profile, 
                     SLOG_prof_t, needed to write the to
                     the disk.

  Modified Input Variables :
. slog - pointer to the SLOG_STREAM where SLOG Display Profile is
         located.

  Modified Output Variables :
. returned value - integer return status.

  Include file needed : 
  slog_profile.h

.N SLOG_RETURN_STATUS
@*/
int SLOG_PROF_Open( SLOG_STREAM  *slog )
{
    slog->prof = ( SLOG_prof_t * ) malloc( sizeof( SLOG_prof_t ) );
    if ( slog->prof == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_Open() - "
                          "malloc() fails, Cannot allocate memory for "
                          "SLOG_prof_t\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    slog->prof->increment  = 10;
    slog->prof->capacity   = slog->prof->increment;
    slog->prof->Nentries   = 0;
    slog->prof->entries    = ( SLOG_intvlinfo_t * )
                               malloc(   slog->prof->capacity
                                       * sizeof( SLOG_intvlinfo_t ) );
    slog->prof->file_loc   = SLOG_fptr_NULL;

    return SLOG_SUCCESS;
}



/*@C
    SLOG_PROF_AddIntvlInfo - Add one display description of an interval to 
                             the SLOG Display Profile table.

  Modified Input Variables :
. slog - pointer to the SLOG_STREAM where SLOG Display Profile Table is
         located.

  Unmodified Input Variables :
. intvltype - index to the interval type.
. classtype - character string for the classtype of the interval
. label     - character string for the label of the interval
. color     - character string for the color of the interval

  Modified Output Variables :
. returned value - integer return status.

  Usage Notes on this subroutine :
     all the input characters strings, classtype, label and color are
     allowed to contain alphanumeric, underscore, blank and tab characters.
     But tab characters, if exist, in the string are all converted to
     blanks for ease of management.

.N SLOG_EXTRA_INTERVAL_INFOS

  Include file needed : 
  slog_profile.h

.N SLOG_RETURN_STATUS
@*/
int SLOG_PROF_AddIntvlInfo(       SLOG_STREAM      *slog,
                            const SLOG_intvltype_t  intvltype,
                            const char *            classtype,
                            const char *            label,
                            const char *            color )
{
    SLOG_prof_t   *profile;
    SLOG_uint32    idx;

    profile = slog->prof;

#if defined( DEBUG )
    if ( profile == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddIntvlInfo() - "
                          "the SLOG_prof_t pointer in slog is NULL\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
#endif

    /*  Check if there is any space left for a new record definition */
    if ( profile->Nentries + 1 > profile->capacity ) {
        profile->capacity += profile->increment;
        profile->entries   = ( SLOG_intvlinfo_t * )
                             realloc( profile->entries,
                                      profile->capacity
                                    * sizeof( SLOG_intvlinfo_t )
                                    );
        if ( profile->entries == NULL ) {
            fprintf( errfile, __FILE__":SLOG_PROF_AddIntvlInfo() - "
                              "realloc() fails, cannot increase the size "
                              "of profile->entries[]\n" );
            fprintf( errfile, "\t""The input Interval Info = [ "
                              fmt_itype_t", %s, %s, %s ] \n",
                              intvltype, classtype, label, color );
            fflush( errfile );
            return SLOG_FAIL;
        }
    }

    
    idx = profile->Nentries;
    SLOG_PROF_Assign( &( profile->entries[ idx ] ),
                      intvltype, classtype, label, color );
    if ( SLOG_PROF_IsDuplicated( profile, &(profile->entries[ idx ]) ) ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddIntvlInfo() - "
                          "there is already a copy of input interval\n" );
        fprintf( errfile, "\t""information with identical keys "
                          "in the SLOG Display Profile table\n" );
        fprintf( errfile, "\t""The input Interval Info = [ "
                          fmt_itype_t", %s, %s, %s ]\n",
                          intvltype, classtype, label, color );
        fflush( errfile );
        return SLOG_FAIL;
    }
    profile->Nentries += 1;

    return SLOG_SUCCESS;
}


/*@C
    SLOG_PROF_Close - Write the SLOG_prof_t onto the disk

  Modified Input Variables :
. slog - pointer to the SLOG_STREAM where SLOG Display Profile Table is
         located.

  Modified Output Variables :
. returned value - integer status code.

  Usage Notes on this subroutine :

.N SLOG_EXTRA_INTERVAL_INFOS

  Include file needed : 
  slog_profile.h

.N SLOG_RETURN_STATUS
@*/
int SLOG_PROF_Close( SLOG_STREAM  *slog )
{
    SLOG_prof_t            *profile;
    SLOG_intvlinfo_t       *info;
    int                     ierr;
    int                     ii;

    /*  Update the content of SLOG_hdr_t to both memory and disk  */
    slog->hdr->fptr2profile = ( SLOG_fptr ) ftell( slog->fd );
    ierr = SLOG_WriteUpdatedHeader( slog );
    if ( ierr != SLOG_SUCCESS ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_Close() - "
                                   "SLOG_WriteUpdatedHeader() fails\n" );
        fflush( errfile );
        return ierr;
    }

    /*  Write the content of SLOG Record Definition Table to the disk  */
    profile = slog->prof;
    
    ierr = bswp_fwrite( &( profile->Nentries ),
                        sizeof( SLOG_uint32 ), 1, slog->fd );
    if ( ierr < 1 ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_Close() - \n"
                           "\t""SLOG_IntvlInfo_WriteToFile() fails at "
                           "writing the number of interval infos to "
                           "table section of logfile\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    for ( ii = 0; ii < profile->Nentries; ii++ ) {
        ierr = SLOG_IntvlInfo_WriteToFile( &( profile->entries[ ii ] ),
                                        slog->fd );
        if ( ierr != SLOG_TRUE ) {
            fprintf( errfile,  __FILE__":SLOG_PROF_Close() - \n"
                               "\t""SLOG_IntvlInfo_WriteToFile() fails at "
                               "%d-th addition of the interval infos to "
                               "table section of logfile\n", ii );
            fflush( errfile );
            return SLOG_FAIL;
        }
    }
  
    return SLOG_SUCCESS;
}



/*@C
    SLOG_PROF_SetExtraNumOfIntvlInfos - Write the starting file pointer 
                                        SLOG_prof_t to the slog's
                                        header and then set the maximum
                                        extra number of Interval 
                                        Description entries
                                        in Display Profile Table.

  Modified Input Variables :
. slog - pointer to the SLOG_STREAM where SLOG Display Profile Table is
         located.

  Unmodifued Input Variables :
. Nentries_extra - Number of extra interval description to be reserved
                   on the disk

  Modified Output Variables :
. returned value - integer status code.

  Usage Notes on this subroutine :

.N SLOG_EXTRA_RECDEFS

  Include file needed : 
  slog_profile.h

.N SLOG_RETURN_STATUS
@*/
int SLOG_PROF_SetExtraNumOfIntvlInfos(       SLOG_STREAM  *slog,
                                       const SLOG_uint32   Nentries_extra )
{
    SLOG_prof_t            *profile;
    SLOG_intvlinfo_t        blank_info = { -1, " ", " ", " " };
    SLOG_uint32             old_profile_capacity;
    int                     ierr;
    int                     ii;

    /*  Update the content of SLOG_hdr_t to both memory and disk  */
    slog->hdr->fptr2profile = ( SLOG_fptr ) ftell( slog->fd );
    ierr = SLOG_WriteUpdatedHeader( slog );
    if ( ierr != SLOG_SUCCESS ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_SetExtraNumOfIntvlInfos() - "
                                   "SLOG_WriteUpdatedHeader() fails\n" );
        fflush( errfile );
        return ierr;
    }

    profile = slog->prof;
    
    /*  Determine the new storage  */
    old_profile_capacity = profile->capacity;
    profile->capacity    = profile->Nentries + Nentries_extra;
    profile->entries = ( SLOG_intvlinfo_t * )
                       realloc( profile->entries,
                                profile->capacity * sizeof( SLOG_intvlinfo_t )
                              );
    if ( profile->entries == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_SetExtraNumOfIntvlInfos() - "
                          "realloc() fails, cannot increase the size "
                          "of profile->entries["fmt_ui32"]\n",
                          profile->capacity );
        fflush( errfile );
        profile->capacity = old_profile_capacity;
        return SLOG_FAIL;
    }

    /*  Write the content of SLOG Record Definition Table to the disk  */

    ierr = bswp_fwrite( &( profile->Nentries ),
                        sizeof( SLOG_uint32 ), 1, slog->fd );
    if ( ierr < 1 ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_SetExtraNumOfIntvlInfos() - \n"
                           "\t""SLOG_IntvlInfo_WriteToFile() fails at "
                           "writing the number of interval descriptors to "
                           "table section of logfile\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    for ( ii = 0; ii < profile->Nentries; ii++ ) {
        ierr = SLOG_IntvlInfo_WriteToFile( &( profile->entries[ ii ] ), 
                                        slog->fd );
        if ( ierr != SLOG_TRUE ) {
            fprintf( errfile,  __FILE__
                               ":SLOG_PROF_SetExtraNumOfIntvlInfos() - \n"
                               "\t""SLOG_IntvlInfo_WriteToFile() fails at the "
                               "%d-th addition of the interval descriptors to "
                               "table section of logfile\n", ii );
            fflush( errfile ); 
            return SLOG_FAIL;
        }
    }

    /*
        Save the location where the extra interval descriptors
        can be inserted
    */

    profile->file_loc = ( SLOG_fptr ) ftell( slog->fd ); 

    /*  Write some blank record definitions to fill up the reserved space.  */

    for ( ii = profile->Nentries; ii < profile->capacity; ii++ ) {
        ierr = SLOG_IntvlInfo_WriteToFile( &blank_info, slog->fd );
        if ( ierr != SLOG_TRUE ) {
            fprintf( errfile,  __FILE__
                               ":SLOG_PROF_SetExtraNumOfIntvlInfos() - \n"
                               "\t""SLOG_IntvlInfo_WriteToFile() fails at "
                               "%d-th addition of the blank descriptor to "
                               "table section of logfile\n", ii );
            fflush( errfile ); 
            return SLOG_FAIL;
        }
    }
  
    return SLOG_SUCCESS;
}



/*@C
    SLOG_PROF_AddExtraIntvlInfo - Add one display description of an
                                  interval to the reserved space of
                                  the SLOG Display Profile table.

  Modified Input Variables :
. slog - pointer to the SLOG_STREAM where SLOG Display Profile Table is
         located.

  Unmodified Input Variables :
. intvltype - index to the interval type.
. classtype - character string for the classtype of the interval
. label     - character string for the label of the interval
. color     - character string for the color of the interval

  Modified Output Variables :
. returned value - integer return status.

  Usage Notes on this subroutine :
     all the input characters strings, classtype, label and color are
     allowed to contain alphanumeric, underscore, blank and tab characters.
     But tab characters, if exist, in the string are all converted to
     blanks for ease of management.

.N SLOG_EXTRA_INTERVAL_INFOS

  Include file needed : 
  slog_profile.h

.N SLOG_RETURN_STATUS
@*/
int SLOG_PROF_AddExtraIntvlInfo(       SLOG_STREAM      *slog,
                                 const SLOG_intvltype_t  intvltype,
                                 const char *            classtype,
                                 const char *            label,
                                 const char *            color )
{
    SLOG_prof_t            *profile;
    SLOG_fptr               file_loc_saved;
    SLOG_uint32             idx;
    int                     ierr;

    profile = slog->prof;

#if defined( DEBUG )
    if ( profile == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddExtraIntvlInfo() - "
                          "the SLOG_prof_t pointer in slog "
                          "is NULL\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
#endif

    if ( profile->Nentries + 1 > profile->capacity ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddExtraIntvlInfo() - \n"
                          "\t""All reserved space in the "
                          "Record Definition Table of the logfile "
                          "has been used up!\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    idx = profile->Nentries;
    SLOG_PROF_Assign( &( profile->entries[ idx ] ),
                      intvltype, classtype, label, color );
    if ( SLOG_PROF_IsDuplicated( profile, &(profile->entries[ idx ]) ) ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddExtraIntvlInfo() - "
                          "there is already a copy of input interval\n " );
        fprintf( errfile, "\t""information with identical keys "
                          "in the SLOG record definition table\n" );
        fprintf( errfile, "\t""The input Interval Info = [ "
                          fmt_itype_t", %s, %s, %s ]\n",
                          intvltype, classtype, label, color );
        fflush( errfile );
        return SLOG_FAIL;
    }
    profile->Nentries += 1;

    /*  Save the current position in the logfile for later restoration  */
    file_loc_saved = ( SLOG_fptr ) ftell( slog->fd );

    /*  Update the number of interval descriptors on the file/disk  */
    ierr = fseek( slog->fd, slog->hdr->fptr2profile, SEEK_SET );
    if ( ierr != 0 ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddExtraIntvlInfo() - "
                          "fseek( fptr2profile ) fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    ierr = bswp_fwrite( &( profile->Nentries ),
                        sizeof( SLOG_uint32 ), 1, slog->fd );
    if ( ierr < 1 ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_AddExtraIntvlInfo() - \n"
                           "\t""Update the number of entries in the record "
                           "definition table on the file fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    /*  Insert the new record definition to the reserved space  */
    ierr = fseek( slog->fd, profile->file_loc, SEEK_SET );
    if ( ierr != 0 ) {
        fprintf( errfile, __FILE__":SLOG_PROF_AddExtraIntvlInfo() - "
                          "fseek( profile->file_loc ) fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    ierr = SLOG_IntvlInfo_WriteToFile( &( profile->entries[idx] ), slog->fd );
    if ( ierr != SLOG_TRUE ) {
        fprintf( errfile,  __FILE__":SLOG_PROF_AddExtraIntvlInfo() - \n"
                           "\t""SLOG_IntvlInfo_WriteToFile() fails at "
                           "%d-th addition of the interval decriptor to "
                           "table section of logfile\n", idx );
        fflush( errfile );
        return SLOG_FAIL;
    }
    fflush( slog->fd );

    /*  Update the file pointer to the reserved space of IntvlInfos in file */
    profile->file_loc = ( SLOG_fptr ) ftell( slog->fd );

    /*  Restore the original file position in the logfile  */
    ierr = fseek( slog->fd, file_loc_saved, SEEK_SET );

    return SLOG_SUCCESS;
}



void SLOG_PROF_Free( SLOG_prof_t *profile )
{
    if ( profile != NULL ) {
        if ( profile->Nentries > 0 && profile->entries != NULL ) {
            free( profile->entries );
            profile->entries = NULL;
        }
        free( profile );
        profile = NULL;
    }
}



int SLOG_PROF_ReadIntvlInfos( SLOG_STREAM  *slog )
{
    SLOG_prof_t   *profile;
    SLOG_intvlinfo_t          *def;
    SLOG_bebits_t           bebits;
    int                     ii;
    int                     ierr;

    slog->prof = ( SLOG_prof_t * ) malloc( sizeof( SLOG_prof_t ) );
    if ( slog->prof == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - "
                          "malloc() fails, Cannot allocate memory for "
                          "SLOG_prof_t\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    /*  Initialize the Record Definition table data structure  */
    slog->prof->increment  = 10;
    slog->prof->capacity   = slog->prof->increment;
    slog->prof->Nentries   = 0;
    slog->prof->entries    = NULL;
    
    /*  Seek to the beginning of the Record Def'n Table in the SLOG file  */
    if ( slog->hdr->fptr2profile == SLOG_fptr_NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - "
                          "slog->hdr->fptr2profile == SLOG_fptr_NULL\n" );
        fprintf( errfile, "\t""SLOG_PROF_Close() may NOT be called\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    ierr = fseek( slog->fd, slog->hdr->fptr2profile, SEEK_SET );
    if ( ierr != 0 ) {
        fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - "
                          "fseek( fptr2profile ) fails\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }

    /*  Read in the data structure  */
    profile = slog->prof;
    
    ierr = bswp_fread( &( profile->Nentries ),
                       sizeof( SLOG_uint32 ), 1, slog->fd );
    if ( ierr < 1 ) {
        fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - \n"
                          "\t""Fails at reading the number of entries "
                          "of the display profile table\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    profile->capacity = profile->Nentries;
    profile->entries  = ( SLOG_intvlinfo_t * )
                        malloc( profile->capacity
                              * sizeof( SLOG_intvlinfo_t ) );
    if ( profile->Nentries > 0 && profile->entries == NULL ) {
        fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - "
                          "malloc() fails for profile->entries[]\n" );
        fflush( errfile );
        return SLOG_FAIL;
    }
    
    for ( ii = 0; ii < profile->Nentries; ii++ ) {
        ierr = SLOG_IntvlInfo_ReadFromFile( &( profile->entries[ ii ] ), 
                                            slog->fd );
        if ( ierr != SLOG_TRUE ) {
            fprintf( errfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - \n"
                              "\t""SLOG_IntvlInfo_ReadFromFile() fails at "
                              "%d-th reading of interval decriptor from "
                              "the table section of logfile\n", ii );
            fflush( errfile ); 
            return SLOG_FAIL;
        }
    }

#if defined( DEBUG )
    fprintf( outfile, __FILE__":SLOG_PROF_ReadIntvlInfos() - Read\n" );
    SLOG_PROF_Print( outfile, profile );
    fflush( outfile );
#endif
  
    return SLOG_SUCCESS;
}



void SLOG_PROF_Print( FILE* fd, const SLOG_prof_t *profile )
{
    int ii;

    for ( ii = 0; ii < profile->Nentries; ii++ ) {
        fprintf( fd, " def[%i] = ", ii );
        SLOG_IntvlInfo_Print( &( profile->entries[ ii ] ), fd );
        fprintf( fd, "\n" );
    }
    fflush( fd );
}



/*  -- Component level supporting subroutines  -- */

        /* SLOG_IntvlInfo_xxxx methods */

void SLOG_IntvlInfo_Print( const SLOG_intvlinfo_t *info, FILE *fd )
{
    fprintf( fd, "[ " fmt_itype_t", %s, %s, %s ]",
                 info->intvltype, info->classtype, info->label, info->color );
}



int SLOG_IntvlInfo_WriteToFile( SLOG_intvlinfo_t *info, FILE *fd )
{
    const char          delimiter = '\t';
    const char          null_char = '\0';
    const char          newline = '\n';
          char          intvltype_str[SLOG_STRING_LEN] = "                    ";
          int           lgth;
          int           ierr;

    /*
    ierr = bswp_fwrite( &( info->intvltype ),
                        sizeof( SLOG_intvltype_t ), 1, fd );
    if ( ierr < 1 ) return SLOG_FALSE;
    */
    /*  snprintf() counts '\0' in the copied char length parameter  */
    lgth = SLOG_STRING_LEN - 2;
    ierr = snprintf( intvltype_str,
                     lgth, "%u", info->intvltype );
    if ( ierr == -1 ) return SLOG_FALSE;
    
    intvltype_str[ lgth+1 ] = delimiter;
    ierr = bswp_fwrite( intvltype_str,
                        sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    info->classtype[ lgth+1 ] = delimiter;
    ierr = bswp_fwrite( &( info->classtype ),
                        sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    info->label[ lgth+1 ] = delimiter;
    ierr = bswp_fwrite( &( info->label ),
                        sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    info->color[ lgth+1 ] = delimiter;
    ierr = bswp_fwrite( &( info->color ),
                        sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    ierr = bswp_fwrite( &newline, sizeof( char ), 1, fd );
    if ( ierr < 1 ) return SLOG_FALSE;

    return SLOG_TRUE;
}



int SLOG_IntvlInfo_ReadFromFile( SLOG_intvlinfo_t *info, FILE *fd )
{
    const char          null_char = '\0';
    const char          delimiter = '\t';
    const char          newline = '\n';
          char          classtype_str[ SLOG_STRING_LEN ];
          int           ierr;

    /*
    ierr = bswp_fread( &( info->intvltype ),
                       sizeof( SLOG_intvltype_t ), 1, fd );
    if ( ierr < 1 ) return SLOG_FALSE;
    */
    ierr = bswp_fread( classtype_str,
                       sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;
    info->intvltype = atoi( classtype_str );

    ierr = bswp_fread( &( info->classtype ),
                       sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    ierr = bswp_fread( &( info->label ),
                       sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    ierr = bswp_fread( &( info->color ),
                       sizeof( char ), SLOG_STRING_LEN, fd );
    if ( ierr < SLOG_STRING_LEN ) return SLOG_FALSE;

    ierr = bswp_fread( &newline, sizeof( char ), 1, fd );
    if ( ierr < 1 ) return SLOG_FALSE;

    return SLOG_TRUE;
}



int SLOG_IntvlInfo_IsKeyEqualTo( const SLOG_intvlinfo_t *info1,
                                 const SLOG_intvlinfo_t *info2 )
{
    return ( info1->intvltype == info2->intvltype );
}

int SLOG_IntvlInfo_IsValueEqualTo( const SLOG_intvlinfo_t *info1,
                                   const SLOG_intvlinfo_t *info2 )
{
    return (    !strcmp( info1->classtype, info2->classtype )
             && !strcmp( info1->label,     info2->label )
             && !strcmp( info1->color,     info2->color ) );
}

int SLOG_IntvlInfo_IsEqualTo( const SLOG_intvlinfo_t *info1,
                              const SLOG_intvlinfo_t *info2 )
{
    return (    SLOG_IntvlInfo_IsKeyEqualTo( info1, info2 )
             && SLOG_IntvlInfo_IsValueEqualTo( info1, info2 ) );
}
